SystemOrganization addCategory: #'Qwaq-Server-Logging'!
SystemOrganization addCategory: #'Qwaq-Server-UI'!

CPanel subclass: #QwaqServerUI
	instanceVariableNames: ''
	classVariableNames: 'InterruptPassword'
	poolDictionaries: ''
	category: 'Qwaq-Server-UI'!

!QwaqServerUI commentStamp: '<historical>' prior: 0!
Common class for Qwaq Server UIs. Subclasses implement specific parts for router, app-server etc.

CProjectMorph open: QwaqServerUI.!

QwaqServerUI defineFields: '<?xml  version="1.0" ?>
<fields>
	<field toGet="logView" toSet="logView:" changeEvent="logViewChanged">logView</field>
	<field toGet="logFile" toSet="logFile:" changeEvent="logFileChanged">logFile</field>
	<field toGet="vncServer" toSet="vncServer:" changeEvent="vncServerChanged">vncServer</field>
	<field toGet="quitOnErrors" toSet="quitOnErrors:" changeEvent="quitOnErrorsChanged">quitOnErrors</field>
	<field toGet="projectMorph" toSet="projectMorph:" changeEvent="projectMorphChanged">projectMorph</field>
	<field toGet="serviceHolder" toSet="serviceHolder:" changeEvent="serviceHolderChanged">serviceHolder</field>
	<field toGet="serviceProvider" toSet="serviceProvider:" changeEvent="serviceProviderChanged">serviceProvider</field>
	<field toGet="realmsPath" toSet="realmsPath:" changeEvent="realmsPathChanged">realmsPath</field>
	<field toGet="usersPath" toSet="usersPath:" changeEvent="usersPathChanged">usersPath</field>
	<field toGet="profiling" toSet="profiling:" changeEvent="profilingChanged">profiling</field>
	<field toGet="profilerProcess" toSet="profilerProcess:" changeEvent="profilerProcessChanged">profilerProcess</field>
	<field toGet="profileLog" toSet="profileLog:" changeEvent="profileLogChanged">profileLog</field>
	<field toGet="defaultServerName" toSet="defaultServerName:" changeEvent="defaultServerNameChanged">defaultServerName</field>
	<field toGet="forumServices" toSet="forumServices:" changeEvent="forumServicesChanged">forumServices</field>
	<field toGet="coalServices" toSet="coalServices:" changeEvent="coalServicesChanged">coalServices</field>
	<field toGet="maxApps" toSet="maxApps:" changeEvent="maxAppsChanged">maxApps</field>
	<field toGet="trafficPane" toSet="trafficPane:" changeEvent="trafficPaneChanged">trafficPane</field>
	<field toGet="dbStore" toSet="dbStore:" changeEvent="dbStoreChanged">dbStore</field>
	<field toGet="vncPort" toSet="vncPort:" changeEvent="vncPortChanged">vncPort</field>
	<field toGet="latencyPane" toSet="latencyPane:" changeEvent="latencyPaneChanged">latencyPane</field>
</fields>'!

!QwaqServerUI class methodsFor: 'LICENSE' stamp: 'ar 4/20/2011 09:46'!
LICENSE
	^'Project OpenQwaq

	Copyright (c) 2005-20011, Teleplace, Inc., All Rights Reserved

	Redistributions in source code form must reproduce the above
	copyright and this condition.

	The contents of this file are subject to the GNU General Public
	License, Version 2 (the "License"); you may not use this file
	except in compliance with the License. A copy of the License is
	available at http://www.opensource.org/licenses/gpl-2.0.php.'! !

!QwaqServerUI class methodsFor: 'accessing' stamp: 'ar 6/12/2007 12:10'!
defaultExtent
	"So the CPanel, the Window, and the partsbin morph agree."
	^ 800@600! !

!QwaqServerUI class methodsFor: 'running' stamp: 'ar 7/1/2010 01:06'!
interruptPassword: password
	"Sets the interrupt password"

	InterruptPassword := TCryptoRandom md5HashMessage: password.
	password atAllPut: $*.
! !

!QwaqServerUI class methodsFor: 'running' stamp: 'ar 8/18/2007 16:16'!
launchAppFrame
	"Launch the application frame"
	| seed root |
	"Errors that happen during launch will just have us die. 
	Not much we can do about it since there is no UI yet."
	SystemErrorHandler default: (SystemErrorHandler handlerBlock:[:err|
		[Smalltalk logError: err description
			inContext: err signalerContext 
			to: self name,'Debug.log'] on: Error do:[].
		Smalltalk quitPrimitive]).
	"Eeek. Reseed islands manually (this should happen automatically!!!!!!)"
	Island allSubInstancesDo:[:each|
		seed := ByteArray new: 128.
		TCryptoRandom gatherEntropy: seed.
		each randomStream key: seed.
	].
	root := CProjectMorph new.
	root noRedraw: true.
	root borderWidth: 0.
	root newProject: self.
	root projectWorld projectMorph: root.
	root openInWorld.
	^root! !

!QwaqServerUI class methodsFor: 'running' stamp: 'ar 7/1/2010 00:25'!
runAsApp
	"Run me as a full screen standalone app"
	^self runAsApp: true! !

!QwaqServerUI class methodsFor: 'running' stamp: 'ar 7/1/2010 00:25'!
runAsApp: interactive
	"Run me as a full screen standalone app.
	When running non-interactive (i.e., as ant task) it is assumed that:
		* The interrupt password has been set
		* The image has the right name already.
	"
	| version passA passB |
	interactive ifTrue:[
		version := UIManager default request: 'Please denote the new version' 
			initialAnswer: QwaqVersion current version.
		version ifNil:[self error: 'A valid version required'].
		QwaqVersion newVersion: version.

		passA := UIManager default requestPassword: 'Please set the interrupt password'.
		passB := UIManager default requestPassword: 'Repeat password to confirm'.
		passA = passB ifFalse:[^self error: 'Password mismatch'].
		self interruptPassword: passA.
		passA := passB := nil.
	].

	InterruptPassword isEmptyOrNil ifTrue:[self error: 'Interrupt password not set'].

	QwaqVersion current seal.
	QwaqParticipantUI flushSurplusPayload.
	WorldState addDeferredUIMessage:[
		World submorphs do:[:each| each delete].
		Preferences disable: #warnIfNoSourcesFile.
		Preferences disable: #warnIfNoChangesFile.
		(interactive not or:[self confirm: 'Save the image?']) ifTrue:[
			AutoStart deinstall. "no autostart please"
			interactive
				ifTrue:[Smalltalk saveAsSuggesting: 'server']
				ifFalse:[Smalltalk snapshot: true andQuit: true].
		].
		self launchAppFrame.
	].! !

!QwaqServerUI methodsFor: 'initialize' stamp: 'ar 6/12/2007 12:13'!
addFloatingPane: aPane
	"Add a QFloatingPane to the UI."
	aPane fillAlpha: 1.0. "no need to be transparent here"
	self add:aPane.
	aPane owner: self.
	aPane hide.! !

!QwaqServerUI methodsFor: 'menu' stamp: 'ar 9/27/2007 11:36'!
addSubMenusTo: aMenu
	"Add sub menus to the menu bar"
	{
		{'File'. self fileMenu}.
		{'Services'. self servicesMenu}.
		{'Tools'. self toolsMenu}.
	} do:[:spec|
		spec last ifNotNil:[
			aMenu add: spec first subMenu: spec last.
		].
	].! !

!QwaqServerUI methodsFor: 'initialize' stamp: 'ar 4/16/2011 17:36'!
bestServerName
	"Find the best guess for the server's name"
	| hostName localAddress |
	(QPropsFile default at: 'OpenQwaq.ServiceIP') ifNotEmpty:[:ip|
		(NetNameResolver addressFromString: ip) ifNotNil:[:addr|
			hostName := NetNameResolver nameForAddress: addr timeout: 5.
		].
		^hostName ifNil:[ip].
	].

	"Service IP not configured.
	Try localHostName / localHostAddress if it doesn't resolve to 127.0.0.1."
	hostName := NetNameResolver localHostName.
	(NetNameResolver addressForName: hostName) = 
		(ByteArray with: 127 with: 0 with: 0 with: 1) ifFalse:[^hostName].

	"Try localHostAddress if it isn't 127.0.0.1"
	localAddress := NetNameResolver localHostAddress.
	localAddress = (ByteArray with: 127 with: 0 with: 0 with: 1) 
			ifFalse:[^NetNameResolver stringFromAddress: localAddress].

	"Try defaultServerName configured during startup"
	^defaultServerName ifNil:[self error: 'Server configuration error: Server has no name']
! !

!QwaqServerUI methodsFor: 'accessing' stamp: 'ar 6/4/2009 14:21'!
coalServices
	"Answer the coalServices of the receiver"
	<bewareOf: #coalServicesChanged>
	^self propertyValueAt: #coalServices! !

!QwaqServerUI methodsFor: 'accessing' stamp: 'ar 6/4/2009 14:21'!
coalServices: aValue
	"Modify the receiver's coalServices"
	^self propertyValueAt: #coalServices put: aValue with: #coalServicesChanged! !

!QwaqServerUI methodsFor: 'accessing' stamp: 'ar 9/3/2008 12:01'!
dbStore
	"Answer the dbStore of the receiver"
	<bewareOf: #dbStoreChanged>
	^self propertyValueAt: #dbStore! !

!QwaqServerUI methodsFor: 'accessing' stamp: 'ar 9/3/2008 12:01'!
dbStore: aValue
	"Modify the receiver's dbStore"
	^self propertyValueAt: #dbStore put: aValue with: #dbStoreChanged! !

!QwaqServerUI methodsFor: 'accessing' stamp: 'ar 8/26/2009 22:59'!
defaultServerName
	"Answer the defaultServerName of the receiver"
	<bewareOf: #defaultServerNameChanged>
	^self propertyValueAt: #defaultServerName! !

!QwaqServerUI methodsFor: 'accessing' stamp: 'ar 8/26/2009 22:59'!
defaultServerName: aValue
	"Modify the receiver's defaultServerName"
	^self propertyValueAt: #defaultServerName put: aValue with: #defaultServerNameChanged! !

!QwaqServerUI methodsFor: 'error handling' stamp: 'ar 6/12/2007 18:38'!
errorReportFor: anError stack: preStack
	"Create an error report"
	^String streamContents:[:s| self errorReportFor: anError stack: preStack on: s].! !

!QwaqServerUI methodsFor: 'error handling' stamp: 'bgf 4/4/2008 14:57'!
errorReportFor: anError stack: preStack on: strm
	"Create an error report"
	| cnt |
 	strm print: Date today; space; print: Time now; cr.
	strm cr.
	strm nextPutAll: (QwaqVersion sysProfile).

	cnt := 0.
	(preStack copyFrom: 1 to: (5 min: preStack size)) do:[:ctx|
		ctx printDetails: strm.	"variable values"
		strm cr.
	].
	strm cr; nextPutAll: '--- The full stack ---'; cr.
	cnt := 0.
	preStack do:[:ctx|
		cnt := cnt + 1.
		cnt = 5 ifTrue: [
			strm nextPutAll: ' - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -'; cr].
		strm print: ctx; cr.  "just class>>selector"
	].
	strm cr; nextPutAll: '--- The transcript output ---'; cr.
	strm nextPutAll: logView panel contents.
! !

!QwaqServerUI methodsFor: 'menu' stamp: 'jcg 11/11/2009 16:00'!
exit
	"Works like the window close button. Really.
	 (Attempting to close or delete the containing system window from 
	 within this menu callback causes grief; the CProjectMorph 
	 does not like to be dismantled from within itself."

	logFile ifNotNil:[logFile close].
	self stopWebServices.
	self stopAppServices.
	self stopServiceProvider.
	serviceHolder stopAll.

	self isSealedApplication ifTrue:[
		"This is the sealed application case."
		Smalltalk quitPrimitive.
	] ifFalse: [
		"Workaround until this can be fixed properly... File/exit from fullScreen
             sometimes leaves a zombie UI appearance covering the Squeak desktop.
		 See Qwaq Mantis 0084."
		window fullScreenMode ifTrue: [
			self toggleFullScreen.
		].

		WorldState addDeferredUIMessage:(MessageSend
			receiver: self window window selector: #closeBoxHit).

		WorldState addDeferredUIMessage:(MessageSend
			receiver: World selector: #restoreMorphicDisplay).
	].! !

!QwaqServerUI methodsFor: 'menu' stamp: 'ar 10/12/2007 11:57'!
fileMenu
	"Answer the file menu for this UI"
	| menu |
	menu := CMenu new.
	menu add: 'Unseal' target: self action: #onUserInterrupt.
	menu add: 'Exit' target: self action: #exit.
	^menu! !

!QwaqServerUI methodsFor: 'accessing' stamp: 'ar 6/12/2007 12:12'!
floatingPanelArea

	"The bounds that QFloatingPanels should be confined to -
	 corresponds to the area inside the menu bar and bottom status panel."
	
	^ (Rectangle origin: (self menu bottomLeft) corner: (self extent)) insetBy: 3.! !

!QwaqServerUI methodsFor: 'accessing' stamp: 'ar 6/12/2007 12:14'!
floatingPanes
	"All the QFloatingPanes used in this UI."
	^ self elements select: [ : e | e isKindOf: QFloatingPane ]! !

!QwaqServerUI methodsFor: 'accessing' stamp: 'ar 9/28/2007 11:31'!
forumServices
	"Answer the forumServices of the receiver"
	<bewareOf: #forumServicesChanged>
	^self propertyValueAt: #forumServices! !

!QwaqServerUI methodsFor: 'accessing' stamp: 'ar 9/28/2007 11:31'!
forumServices: aValue
	"Modify the receiver's forumServices"
	^self propertyValueAt: #forumServices put: aValue with: #forumServicesChanged! !

!QwaqServerUI methodsFor: 'error handling' stamp: 'ar 6/12/2007 18:36'!
handleError: anError
	"Handle an otherwise unhandled error. This method is called concurrently 
	and care must be taken to schedule it properly."
	| process preStack |
	process := Processor activeProcess.
	"Cover the stack trace *before* the actual error handling to avoid nasty effects due to the currently active process being terminated before we get around to handling the signal."
	preStack := anError signalerContext stackOfSize: 1000.
	self signal: #error withArguments: {anError. process. preStack}.
	"Make sure we don't kill Morphic with the #suspend below"
	Project spawnNewProcessIfThisIsUI: process.
	process suspend.! !

!QwaqServerUI methodsFor: 'error handling' stamp: 'ar 8/20/2008 11:53'!
handleError: anError inProcess: aProcess stack: preStack
	"Handle an error in the given process"
	| report |
	<on: error>
	[report := self errorReportFor: anError stack: preStack.
	QLogger logError: anError details: report from: self.
	Transcript cr; show: report.
	aProcess terminate] on: Error do:[:ex| ex return].
	quitOnErrors ifTrue:[self exit].! !

!QwaqServerUI methodsFor: 'error handling' stamp: 'ar 6/14/2007 14:08'!
handleUserInterrupt
	"Handle a user interrupt (CMD-period)"
	Transcript cr; show: 'Caught interrupt:'.
	Transcript cr; show: thisContext longStack.
	self signal: #userInterrupt.! !

!QwaqServerUI methodsFor: 'initialize' stamp: 'ar 9/3/2010 11:00'!
initialize
	"Initialize the receiver"
	super initialize.
	self initializeMemorySettings.
	UTF8TextConverter strictUtf8Conversions: false.
	QDocumentType initialize. "make sure the psuedo-mime table is fresh."
	serviceHolder := QServiceHolder new.
	realmsPath := '../realms'.
	quitOnErrors := false. "by default don't quit"
	maxApps := 40.

	self define: #logView as: QLogView new.
	self define: #trafficPane as: QTrafficPane new.
	self define: #latencyPane as: QLatencyPane new.
	
	self isSealedApplication ifTrue:[
		SystemErrorHandler default: self.
		Sensor installInterruptWatcher:[self userInterruptWatcher].
		Smalltalk installLowSpaceWatcher:[self lowSpaceWatcher].
	].
	Transcript addDependent: self. "get log messages here"

	Smalltalk platformName = 'unix' ifTrue:[
		logFile := [FileStream oldFileNamed:'/dev/stdout'] on: Error do:[:ex| ex return: nil].
	].

	OSProcessAccessor errorLog: QLogger.

	profiling := false.
	profileLog := 'profile.log'.
	TTrafficCounter reset.
! !

!QwaqServerUI methodsFor: 'initialize' stamp: 'ar 3/25/2008 12:32'!
initializeMemorySettings
	"Initialize the memory and GC settings to be more in line with QF requirements"

	"The following settings affect the rate incremental GCs and tenuring"

		"Limit incremental GC activity to run every 40k allocations"
		Smalltalk vmParameterAt: 5 put: 40000. "allocations between GCs (default: 4000)"
		"Limit tenuring threshold to only tenure w/ > 10k survivors"
		Smalltalk vmParameterAt: 6 put: 10000. "tenuring threshold (default: 2000)"

	"These settings affect overall memory usage"

		"Only give memory back to the OS when we have more than 16MB free"
		Smalltalk vmParameterAt: 24 put: 16*1024*1024. "default 8MB"
		"Try to keep 8MB headroom at all times"
		Smalltalk vmParameterAt: 25 put: 8*1024*1024. "default 4MB"

	"These settings describe what to do when we're close to running out of free space"

		"Tell the VM that we'd rather grow than spinning in tight GC loops"
		Smalltalk gcBiasToGrow: true. "default: false"
		"Tell the VM to do a fullGC for good measure if the above growth exceeded 16MB"
		Smalltalk gcBiasToGrowLimit: 16*1024*1024. "default: 0"
! !

!QwaqServerUI methodsFor: 'menu' stamp: 'bgf 8/24/2007 22:00'!
installMenu
	"Create and install the menu"
	self menu ifNotNil:[self menu destroy].
	self menu: CMenuBar new.
	self menu floating: false.
	self menu borderWidth: 1; borderStyle: #raised.
	self menu hResizing: #spaceFill.
	self addSubMenusTo: self menu.

	QStyles installMenuFontInMenu: self menu.
! !

!QwaqServerUI methodsFor: 'testing' stamp: 'jdm 12/10/2007 06:47'!
isSealedApplication
	"Answer whether we're currently running as a sealed application."
	^ QwaqVersion current isSealedApplication! !

!QwaqServerUI methodsFor: 'accessing' stamp: 'ar 7/29/2010 18:16'!
latencyPane
	"Answer the latencyPane of the receiver"
	<bewareOf: #latencyPaneChanged>
	^self propertyValueAt: #latencyPane! !

!QwaqServerUI methodsFor: 'accessing' stamp: 'ar 7/29/2010 18:16'!
latencyPane: aValue
	"Modify the receiver's latencyPane"
	^self propertyValueAt: #latencyPane put: aValue with: #latencyPaneChanged! !

!QwaqServerUI methodsFor: 'initialize' stamp: 'ar 5/14/2011 14:15'!
licenseFileContents
	"Answers the contents of the license file"
	| file data |
	file := [FileStream readOnlyFileNamed: 
			(FileDirectory default pathFromURI: '../../admin/OpenQwaq-license')
	] on: Error do:[:ex|
		(ex isKindOf: FileDoesNotExistException) ifTrue:[^'No license file found'].
		^ex description
	].
	[data := file binary contents asString] ensure:[file close].
	^data! !

!QwaqServerUI methodsFor: 'menu' stamp: 'bgf 2/24/2009 15:09'!
linkItem:aMenuItem toTool:aPanel

	"Make the given menu item checkmark track whether or not the given tool is visible."

	aMenuItem checked: (aPanel visible).
	self runScript: #toggleItem:forTool: withArguments: {aMenuItem . aPanel} when: {aPanel . #visibleChanged}.! !

!QwaqServerUI methodsFor: 'accessing' stamp: 'ar 6/12/2007 12:01'!
logFile
	"Answer the logFile of the receiver"
	<bewareOf: #logFileChanged>
	^self propertyValueAt: #logFile! !

!QwaqServerUI methodsFor: 'accessing' stamp: 'ar 6/12/2007 12:01'!
logFile: aValue
	"Modify the receiver's logFile"
	^self propertyValueAt: #logFile put: aValue with: #logFileChanged! !

!QwaqServerUI methodsFor: 'accessing' stamp: 'ar 6/12/2007 12:00'!
logView
	"Answer the logView of the receiver"
	<bewareOf: #logViewChanged>
	^self propertyValueAt: #logView! !

!QwaqServerUI methodsFor: 'accessing' stamp: 'ar 6/12/2007 12:00'!
logView: aValue
	"Modify the receiver's logView"
	^self propertyValueAt: #logView put: aValue with: #logViewChanged! !

!QwaqServerUI methodsFor: 'status' stamp: 'ar 7/2/2007 11:36'!
logViewCharacterLimit
	"Answer the max. number of characters we want to show in the log view.
	When this number is exceeded the first half of the log view contents will be deleted."
	^50000! !

!QwaqServerUI methodsFor: 'error handling' stamp: 'ar 6/14/2007 14:20'!
lowSpaceWatcher
	"do nothing"! !

!QwaqServerUI methodsFor: 'accessing' stamp: 'ar 2/29/2008 10:21'!
mainView
	"QFloatingPanes are very unhappy inside an owner without a mainView property."
	^nil! !

!QwaqServerUI methodsFor: 'accessing' stamp: 'ar 10/17/2007 11:09'!
maxApps
	"Answer the maxApps of the receiver"
	<bewareOf: #maxAppsChanged>
	^self propertyValueAt: #maxApps! !

!QwaqServerUI methodsFor: 'accessing' stamp: 'ar 10/17/2007 11:09'!
maxApps: aValue
	"Modify the receiver's maxApps"
	^self propertyValueAt: #maxApps put: aValue with: #maxAppsChanged! !

!QwaqServerUI methodsFor: 'accessing' stamp: 'jcg 11/11/2009 15:24'!
netVidServices
	^serviceHolder netVidServices! !

!QwaqServerUI methodsFor: 'status' stamp: 'ar 7/2/2007 11:26'!
onAppendEntry: aString with: anEvent
	"This is dog stupid and cheap... but it works."
	<on: appendEntry>
	anEvent withDroppedEventsDo:[:evt|
		logView panel selectAt: logView panel contents size+1.
		logView panel paste: evt arguments first select: false.
		logFile ifNotNil:[logFile nextPutAll: (evt arguments first copyReplaceAll: String cr with: String crlf)].
		logView panel contents size > self logViewCharacterLimit ifTrue:[
			logView panel contents: (logView panel contents last: self logViewCharacterLimit // 2).
		].
	].! !

!QwaqServerUI methodsFor: 'events' stamp: 'bgf 6/15/2008 10:35'!
onExtentChanged
	"The size of the panel has changed.  We do our layout management here."
	| box |
	<on: extentChanged>
	box := self localBounds.
	self menu topLeft: box topLeft.
	self menu width: box width.

	"Position the floating panes."
	self floatingPanes do: [ :p| p alignToOwner ].! !

!QwaqServerUI methodsFor: 'events' stamp: 'ar 5/22/2008 14:54'!
onOpen
	"Do various things after the ui is visible"
	<on: open>
	projectMorph ifNotNil:[
		WorldState addDeferredUIMessage:[
			projectMorph noRedraw: false.
			projectMorph changed.
			self signal: #completeAfterShowingScreen].
		self waitUntil: #completeAfterShowingScreen].


	[Python initialize] on: Error do:[:ex|
		Transcript cr; show: 'ERROR INITIALIZING PYTHON'.
		Transcript cr; show: ex description.
		Transcript cr; show: ex signalerContext longStack.
	].

	self processCommandLine.
! !

!QwaqServerUI methodsFor: 'events' stamp: 'ar 8/26/2008 18:40'!
onUserInterrupt
	"Handle a user interrupt"
	| pass hash |
	<on: userInterrupt>
	pass := UIManager default requestPassword: 'Enter the interrupt password:'.
	pass ifNil:[^nil].
	hash := TCryptoRandom md5HashMessage: pass.
	hash = InterruptPassword ifFalse:[^self inform:'Incorrect password'].
	WorldState addDeferredUIMessage:[projectMorph collapse].! !

!QwaqServerUI methodsFor: 'profiler' stamp: 'ar 8/30/2007 13:46'!
printProcess: aProcess
	"Print aProcess in a useful form"
	| ctx |
	^String streamContents:[:s|
		s nextPutAll: '[', aProcess priority, '] ', aProcess class name.
		s nextPutAll: '(', aProcess name, ') '.
		(ctx := aProcess suspendedContext) ifNotNil:[
			[ctx home sender == nil] whileFalse:[ctx := ctx home sender].
			s print: ctx.
		].
	].! !

!QwaqServerUI methodsFor: 'initialize' stamp: 'ar 6/12/2007 18:27'!
processCommandLine
	"Process command line arguments"
	| index |
	self isSealedApplication ifFalse:[^nil]. "only process command line for sealed apps"
	index := 0.
	[index := self processCommandLine: index.
	index == nil] whileFalse.
! !

!QwaqServerUI methodsFor: 'initialize' stamp: 'ar 6/16/2010 14:24'!
processCommandLine: idx
	"Process command line arguments"
	| attr index arg |
	attr := Smalltalk getSystemAttribute: (index := idx + 1).
	attr ifNil:[^nil].
	attr caseOf:{
		['-configFile:'] -> [
			arg :=  Smalltalk getSystemAttribute: (index := index + 1).
			self processConfigFile: arg.
		].
		['-vncPort:'] -> [
			arg :=  Smalltalk getSystemAttribute: (index := index + 1).
			vncPort := arg asNumber.
		].
		['-defaultServerName:'] -> [ | stream |
			arg :=  Smalltalk getSystemAttribute: (index := index + 1).
			stream := arg readStream skipSeparators.
			stream atEnd ifFalse:[
				defaultServerName := (stream upToAnyIn: CharacterSet separators) allButLast.
			].
		].
		['-startServiceProvider'] 	-> [self startServiceProvider].
		['-startRouterServices'] 	-> [self startRouterServices].
		['-startAppServices'] 	-> [self startAppServices].
		['-startVideoServices'] 	-> [self startNetVidServices].
		"XXXXX: support both options for now, eventually just keep the webcast option."
		['-startBroadcastServices'] 	-> [self startWebcastServices].
		['-startWebcastServices'] 	-> [self startWebcastServices].
		['-startWebServices']		-> [self startWebServices].
		['-startClientProxyServices'] 	-> [self startProxyServices].
	} otherwise:[].
	^index! !

!QwaqServerUI methodsFor: 'initialize' stamp: 'ar 4/16/2011 19:00'!
processConfigFile: fileName
	"Process configuration options from the given file"
	| propsFile active backlog |
	propsFile := QPropsFile on: (QCachedFile on: fileName).
	QPropsFile default: propsFile.
	QServerConstants setupWithPrefix: 'OpenQwaq.ServerConstants.'.
	self setupThreadedODBC.
	serviceHolder serviceIP: (propsFile at: 'OpenQwaq.ServiceIP').

	self setupLogs.

	"Set the maximum backlog allowed before we kill a channel."
	backlog := propsFile at: 'OpenQwaq.Network.MaxChannelBacklog' ifAbsent: [nil].
	backlog ifNotNil: [backlog := backlog asNumber].
	TMessageRelay maxChannelBacklog: backlog.

	"Cloud support initialization."
	(((propsFile at: 'OpenQwaq.Cloud.Enabled' ifAbsent: ['false']) = 'true') and: [Smalltalk platformName ~= 'Win32']) ifTrue: [EC2 initializeFromConfiguration: propsFile].

	"VNC options"
	"Do this last. Otherwise, since the VNC port is set by a later command-line option than the one that invokes this config file processing, any long-ish operations done after the startVNCServer script is started could cause that port to be needed before it is set."
	active := (propsFile at: 'OpenQwaq.VNC.Active' ifAbsent:['false']) = 'true'.
	active ifTrue:[self startScript: #startVNCServer]
! !

!QwaqServerUI methodsFor: 'accessing' stamp: 'ar 8/30/2007 12:36'!
profileLog
	"Answer the profileLog of the receiver"
	<bewareOf: #profileLogChanged>
	^self propertyValueAt: #profileLog! !

!QwaqServerUI methodsFor: 'accessing' stamp: 'ar 8/30/2007 12:36'!
profileLog: aValue
	"Modify the receiver's profileLog"
	^self propertyValueAt: #profileLog put: aValue with: #profileLogChanged! !

!QwaqServerUI methodsFor: 'accessing' stamp: 'ar 8/30/2007 12:10'!
profilerProcess
	"Answer the profilerProcess of the receiver"
	<bewareOf: #profilerProcessChanged>
	^self propertyValueAt: #profilerProcess! !

!QwaqServerUI methodsFor: 'accessing' stamp: 'ar 8/30/2007 12:10'!
profilerProcess: aValue
	"Modify the receiver's profilerProcess"
	^self propertyValueAt: #profilerProcess put: aValue with: #profilerProcessChanged! !

!QwaqServerUI methodsFor: 'accessing' stamp: 'ar 8/30/2007 12:10'!
profiling
	"Answer the profiling of the receiver"
	<bewareOf: #profilingChanged>
	^self propertyValueAt: #profiling! !

!QwaqServerUI methodsFor: 'accessing' stamp: 'ar 8/30/2007 12:10'!
profiling: aValue
	"Modify the receiver's profiling"
	^self propertyValueAt: #profiling put: aValue with: #profilingChanged! !

!QwaqServerUI methodsFor: 'accessing' stamp: 'ar 6/12/2007 19:15'!
projectMorph
	"Answer the projectMorph of the receiver"
	<bewareOf: #projectMorphChanged>
	^self propertyValueAt: #projectMorph! !

!QwaqServerUI methodsFor: 'accessing' stamp: 'ar 6/12/2007 19:15'!
projectMorph: aValue
	"Modify the receiver's projectMorph"
	^self propertyValueAt: #projectMorph put: aValue with: #projectMorphChanged! !

!QwaqServerUI methodsFor: 'accessing' stamp: 'ar 6/12/2007 18:17'!
quitOnErrors
	"Answer the quitOnErrors of the receiver"
	<bewareOf: #quitOnErrorsChanged>
	^self propertyValueAt: #quitOnErrors! !

!QwaqServerUI methodsFor: 'accessing' stamp: 'ar 6/12/2007 18:17'!
quitOnErrors: aValue
	"Modify the receiver's quitOnErrors"
	^self propertyValueAt: #quitOnErrors put: aValue with: #quitOnErrorsChanged! !

!QwaqServerUI methodsFor: 'events' stamp: 'ar 7/16/2008 11:52'!
readyForEvents
	^true! !

!QwaqServerUI methodsFor: 'accessing' stamp: 'ar 7/6/2007 11:06'!
realmsPath
	"Answer the realmsPath of the receiver"
	<bewareOf: #realmsPathChanged>
	^self propertyValueAt: #realmsPath! !

!QwaqServerUI methodsFor: 'accessing' stamp: 'ar 7/6/2007 11:06'!
realmsPath: aValue
	"Modify the receiver's realmsPath"
	^self propertyValueAt: #realmsPath put: aValue with: #realmsPathChanged! !

!QwaqServerUI methodsFor: 'profiler' stamp: 'ar 8/30/2007 13:55'!
reportProfile: aBag on: aStream
	| totalTicks threshold ticks percent prefix tickCount |
	totalTicks := aBag size.
	threshold := totalTicks * 95 // 100. "95%"
	prefix := '[', Date today yyyymmdd, ' ', Time now print24, '] '.
	tickCount := 0.
	aBag sortedCounts do:[:assoc|
		ticks := assoc key.
		tickCount < threshold ifTrue:[
			percent := ticks * 100 // totalTicks.
			percent < 1 ifTrue:[percent := (ticks * 10000 // totalTicks) * 0.01].
			aStream crlf; nextPutAll: prefix, percent, '% ', (self printProcess: assoc value).
		].
		tickCount := tickCount + ticks.
	].
	aStream crlf.
! !

!QwaqServerUI methodsFor: 'accessing' stamp: 'jcg 11/11/2009 15:25'!
routerServices
	^serviceHolder routerServices! !

!QwaqServerUI methodsFor: 'profiler' stamp: 'ar 8/30/2007 14:13'!
runProfiler
	"Run a cheap version of QSystemProfiler which only looks at the process name"
	| semaphore ticks tallyStart process tallyTicks lastReport bag dir file |
	semaphore := Semaphore new.
	Smalltalk profileSemaphore: semaphore.
	ticks := Smalltalk interruptChecksPerMSec // 10.
	bag := Bag new.
	lastReport := Time totalSeconds.
	[profiling] whileTrue:[
		tallyStart := Smalltalk highResClock.
		Smalltalk profileStart: ticks. "run for n ticks"
		semaphore wait.
		process := Smalltalk profileSample.
		Smalltalk profileStart: 0. "stop counting"
		tallyTicks := Smalltalk highResClock - tallyStart.
		process ifNotNil:[bag add: process withOccurrences: tallyTicks].
		"Report every five minutes"
		Time totalSeconds - lastReport > 300 ifTrue:[
			dir := QLogger logDirectory ifNil:[FileDirectory default].
			file := [dir fileNamed: profileLog] on: Error do:[:ex| ex return: nil].
			file ifNil:[^nil].
			file setToEnd.
			self reportProfile: bag on: file.
			file close.
			lastReport := Time totalSeconds.
			bag := Bag new.
		].
	].
	Smalltalk profileSemaphore: nil.
	profilerProcess := nil.
! !

!QwaqServerUI methodsFor: 'accessing' stamp: 'jcg 11/11/2009 13:16'!
serviceHolder
	"Answer the serviceHolder of the receiver"
	<bewareOf: #serviceHolderChanged>
	^self propertyValueAt: #serviceHolder! !

!QwaqServerUI methodsFor: 'accessing' stamp: 'jcg 11/11/2009 13:16'!
serviceHolder: aValue
	"Modify the receiver's serviceHolder"
	^self propertyValueAt: #serviceHolder put: aValue with: #serviceHolderChanged! !

!QwaqServerUI methodsFor: 'accessing' stamp: 'ar 6/4/2009 13:48'!
serviceProvider
	"Answer the serviceProvider of the receiver"
	<bewareOf: #serviceProviderChanged>
	^self propertyValueAt: #serviceProvider! !

!QwaqServerUI methodsFor: 'accessing' stamp: 'ar 6/4/2009 13:48'!
serviceProvider: aValue
	"Modify the receiver's serviceProvider"
	^self propertyValueAt: #serviceProvider put: aValue with: #serviceProviderChanged! !

!QwaqServerUI methodsFor: 'menu' stamp: 'hrs 1/31/2010 15:38'!
servicesMenu
	"Answer the services menu for this UI"
	| menu item |
	menu := CMenu new.
	item := menu add: 'Start Service Provider' target: self action: #toggleServiceProvider.
	item enabled: false.
	item tag: #startServiceProvider.
	menu addSeparator.
	item := menu add: 'Start Router Services' target: self action: #toggleRouterServices.
	item enabled: false.
	item tag: #startRouterServices.
	item := menu add: 'Start Application Services' target: self action: #toggleAppServices.
	item enabled: false.
	item tag: #startAppServices.
	item := menu add: 'Start NetVid Services' target: self action: #toggleNetVidServices.
	item enabled: false.
	item tag: #startNetVidServices.
	item := menu add: 'Start Webcast Services' target: self action: #toggleWebcastServices.
	item enabled: false.
	item tag: #startWebcastServices.
	item := menu add: 'Start Web (QRL) Services' target: self action: #toggleWebServices.
	item enabled: false.
	item tag: #startWebServices.
	item := menu add: 'Start Client Proxy Services' target: self action: #toggleProxyServices.
	item enabled: false.
	item tag: #startProxyServices.
	menu addSeparator.
	item := menu add: 'Start Profiler' target: self action: #toggleProfiler.
	item tag: #startProfiler.
	^menu! !

!QwaqServerUI methodsFor: 'initialize' stamp: 'ar 7/29/2010 18:17'!
setupCostume
	| w |
	super setupCostume.

	self clipping: true.
	self installMenu.
	self add: self menu.

	self extent: (self class defaultExtent).
	w := self window window.
	w ifNotNil:[w extent: (self class defaultExtent)].
	self color: (Color gray).

	"All floating panes start hidden so we don't eat CPU unnecessarily"
	self addFloatingPane: logView.
	self addFloatingPane: trafficPane.
	self addFloatingPane: latencyPane.

	self menu fillAlpha: 0.85.

! !

!QwaqServerUI methodsFor: 'initialize' stamp: 'ar 4/16/2011 17:36'!
setupLogs
	"Process configuration options from the given file"
	| logDir |
	QLogger reset.
	QLogger default: QServerLogger new.
	QLogger default serverName: self bestServerName.
	logDir := QPropsFile default at: 'OpenQwaq.LogDir'.
	logDir ifNotEmpty:["don't clobber the existing log dir unless it's specified"
		QLogger logDirectory: (FileDirectory on: (FileDirectory pathFromURI: logDir)).
	].

	QLogger logInfo: 	'Server starting'.
	QLogger logInfo: 'Server Version: ', self versionFileContents.
	QLogger logInfo: 'Image Version: ',  QwaqVersion current infoString.
	QLogger logInfo: 'Server License: ', self licenseFileContents.
! !

!QwaqServerUI methodsFor: 'initialize' stamp: 'ar 4/16/2011 19:00'!
setupThreadedODBC
	"Set up the threaded ODBC if required"

	| dbStats |
	(QPropsFile default at: 'OpenQwaq.ODBC.Threading') = 'false' ifTrue:[^self].

	ODBCConnection connectionClass: ODBCThreadedConnection.
	ODBCThreadedConnection eventLoop: ODBCEventLoop new.
	dbStats := TWindowedStatistics forWindowDuration: 60000..
	dbStats tagName: #ODBC.
	ODBCThreadedConnection latencyStats: dbStats.
	latencyPane startWatching: dbStats.! !

!QwaqServerUI methodsFor: 'services' stamp: 'ar 4/16/2011 19:00'!
startActivityLog
	| dsn odbc |
	(dsn := QPropsFile default at: 'OpenQwaq.ActivityLog.DSN') ifNotEmpty:[
		"Use the provided data source for logging"
		odbc := (Smalltalk 
			at: #ODBCConnection 
			ifAbsent:[self error:'ODBC support is missing']) connectionClass new
				initializeDsn: dsn 
				user: (QPropsFile default at: 'OpenQwaq.ActivityLog.User') 
				password: (QPropsFile default at: 'OpenQwaq.ActivityLog.Password').
		[odbc connect] on: Error do:[:ex| QLogger logError: ex].
		QLogger activityLog setupActivityQuery: (QPropsFile default at: 'OpenQwaq.ActivityLog.Insert').
		QLogger activityLog odbc: odbc.
		QLogger logInfo: 'Starting Activity Logger (dsn: ', dsn, ')'.
	].
! !

!QwaqServerUI methodsFor: 'services - delegated' stamp: 'jcg 11/12/2009 12:46'!
startAppServices
	"Start the app-services (for mananging TMS apps)."
	serviceHolder startAppServices.
	(self menu itemTagged: #startAppServices) checked: true.
	! !

!QwaqServerUI methodsFor: 'services' stamp: 'ar 4/16/2011 17:36'!
startDefaultStorage
	"Fallback storage"
	QLogger logWarning: 'Using default storage'.
	dbStore := QLocalStorageDB newFrom: QPropsFile default withPrefix: 'OpenQwaq.SP.'.
	dbStore activate.
! !

!QwaqServerUI methodsFor: 'services' stamp: 'ar 4/16/2011 17:36'!
"LDAP options (need to come before SP)"
startLdapStorage
	| active dbClass |
	active := (QPropsFile default at: 'OpenQwaq.Ldap.Active' ifAbsent:['false']) = 'true'.
	active ifTrue:[
		dbClass := Smalltalk at: #QSStorageLDAP ifAbsent:[nil].
		dbClass ifNil:[self error: 'LDAP support is missing'].
		dbStore := dbClass newFrom: QPropsFile default  withPrefix: 'OpenQwaq.'.
		dbStore activate.
	].

! !

!QwaqServerUI methodsFor: 'services - delegated' stamp: 'jcg 11/11/2009 15:07'!
startNetVidServices
	"Start the NetVid (streaming-video) services"
	serviceHolder startNetVidServices.
	(self menu itemTagged: #startNetVidServices) checked: true.
! !

!QwaqServerUI methodsFor: 'services' stamp: 'ar 4/16/2011 17:37'!
startOdbcStorage
	"ODBC options (need to come before SP)"
	| active dbClass |
	active := (QPropsFile default at: 'OpenQwaq.SP.ODBC.Active' ifAbsent:['false']) = 'true'.
	active ifTrue:[
		dbClass := Smalltalk at: #QSStorageODBC ifAbsent:[nil].
		dbClass ifNil:[self error: 'ODBC support is missing'].
		dbStore := dbClass newFrom: QPropsFile default withPrefix: 'OpenQwaq.SP.'.
		dbStore activate.
	].
! !

!QwaqServerUI methodsFor: 'profiler' stamp: 'ar 8/30/2007 12:38'!
startProfiling
	"Start the profiler"
	profiling := true.
	profilerProcess := [self runProfiler] forkAt: Processor timingPriority - 1.
	(self menu itemTagged: #startProfiler) checked: true.! !

!QwaqServerUI methodsFor: 'services - delegated' stamp: 'hrs 2/1/2010 16:49'!
startProxyServices
	"Start the proxy services (for mananging foreign client proxies)."
	serviceHolder startProxyServices.
	(self menu itemTagged: #startProxyServices) checked: true.
	! !

!QwaqServerUI methodsFor: 'services - delegated' stamp: 'jcg 11/11/2009 15:17'!
startRouterServices
	"Start the NetVid (streaming-video) services"
	serviceHolder startRouterServices.
	(self menu itemTagged: #startRouterServices) checked: true.
! !

!QwaqServerUI methodsFor: 'services' stamp: 'ar 4/16/2011 19:00'!
startServiceProvider
	"Start the front-end forum services"

	serviceProvider ifNotNil:[^self]. "already running"
	QLogger default baseName ifNil:[QLogger default baseName: 'forums'].

	realmsPath := QPropsFile default at: 'OpenQwaq.SP.RealmsPath' ifAbsent:['realms'].
	usersPath := QPropsFile default at: 'OpenQwaq.SP.UsersPath' ifAbsent:['users'].

	self startOdbcStorage.
	self startLdapStorage.
	dbStore ifNil:[self startDefaultStorage].
	self startActivityLog.

	serviceHolder startInternalHttpDispatcher.
	serviceHolder startClientDispatcher.

	"Create the service provider"
	QLogger logInfo: 'Starting service provider'.
	QServiceProvider reset.
	serviceProvider := QServiceProvider default.
	serviceProvider poolName: (QPropsFile default at: 'OpenQwaq.SP.PoolName').
	serviceProvider installIn: serviceHolder clientDispatcher.
	serviceProvider streamManager: 
		(serviceHolder startStreamManagerOn: QServerConstants serviceProviderPort).
	serviceProvider defaultServerName: defaultServerName.

	serviceProvider realmsBasePath: (FileDirectory default pathFromURI: realmsPath).
	serviceProvider userBasePath: (FileDirectory default pathFromURI: usersPath).
	serviceProvider dbStore: dbStore.

	latencyPane startWatching: serviceProvider latencyStats.

	QClientActivityLoggerToODBC setupWithPrefix: 'OpenQwaq.SP.'.
	QExitQuizLoggerToODBC setupWithPrefix: 'OpenQwaq.SP.'.

	serviceProvider future recover.

	(self menu itemTagged: #startServiceProvider) checked: true.

	forumServices := QForumServices new.
	forumServices serviceProvider: serviceProvider.
	forumServices installServicesIn: serviceHolder internalHttpDispatcher.

	coalServices := QCoalServices new.
	coalServices serviceProvider: serviceProvider.
	coalServices installServicesIn: serviceHolder internalHttpDispatcher.

	QwaqMail mailPropFile: QPropsFile default.

	QLogger logInfo: 'Started service provider'.
! !

!QwaqServerUI methodsFor: 'services' stamp: 'ar 4/16/2011 17:37'!
startVNCServer
	"Launch the VNC server for this image"
	QLogger logInfo: 'Starting VNC (port: ', vncPort, ')'.
	self startVncServer: vncPort asNumber.
	RFBServer setFullPassword: 
		(QPropsFile default at: 'OpenQwaq.VNC.Password') copy.! !

!QwaqServerUI methodsFor: 'vnc support' stamp: 'sn 8/14/2012 23:32'!
startVncServer: port
	"Launch the VNC server on the given port"
	| interface |
	self stopVncServer.
	interface := "(QPropsFile default at: 'OpenQwaq.ServiceIP')" '192.168.1.36' 
		ifEmpty:[nil]
		ifNotEmpty:[:if| NetNameResolver addressForName: if].
	RFBServer logging: false.
	RFBServer allowZRLE: true. "enable ZRLE so loopback will work with our clients"

	[vncServer := RFBServer new start: port interface: interface] on: Error do:[:ex| 
		Transcript cr; show: ex description.
		^self stopVncServer].
	(self menu itemTagged: #enableVNC) checked: true.! !

!QwaqServerUI methodsFor: 'services' stamp: 'jcg 11/13/2009 18:26'!
startWebServices
	"Start the service provider"

	QWebServices isRunning ifTrue: [^self]. "already running"
	QLogger default baseName ifNil:[QLogger default baseName: 'webserver'].

	false ifTrue:[
		"XXXX: Do not start the HTTP service dispatcher automatically.
		If we have a service dispatcher (from startForumServices) then using it is fine
		but creating one if there is none is subject to problems when we are running
		a sandwiched server where the http dispatcher of the web services slice will 
		likely conflict with the http dispatcher of the SP services slice. Since this can 
		lead to hard to find problems, I'd rather have us fall over right away when this 
		happens instead of wasting hours and hours on trying to find out what is going 
		wrong."
		serviceHolder startInternalHttpDispatcher.
	].

	"defaults for mumbleWebHostName might have been set interactively."
	serviceHolder startExternalHttpDispatcher.
	QWebServices 
		start: serviceProvider 
		administrativeDispatcher: serviceHolder internalHttpDispatcher 
		publicDispatcher: serviceHolder externalHttpDispatcher.
	(self menu itemTagged: #startWebServices) checked: true.! !

!QwaqServerUI methodsFor: 'services - delegated' stamp: 'jcg 11/25/2009 01:07'!
startWebcastServices
	"Start the RTSP broadcast services"
	serviceHolder startWebcastServices.
	(self menu itemTagged: #startWebcastServices) checked: true.			
! !

!QwaqServerUI methodsFor: 'services - delegated' stamp: 'jcg 11/12/2009 12:48'!
stopAppServices
	"Stop the app-services (for mananging TMS apps)."
	serviceHolder stopAppServices.
	(self menu itemTagged: #startAppServices) checked: false.
! !

!QwaqServerUI methodsFor: 'services - delegated' stamp: 'jcg 11/11/2009 15:11'!
stopNetVidServices
	"Stop the net-vid (streaming video) server"
	serviceHolder stopNetVidServices.
	(self menu itemTagged: #startNetVidServices) checked: false.
! !

!QwaqServerUI methodsFor: 'profiler' stamp: 'ar 8/30/2007 12:38'!
stopProfiling
	"Stop the profiler"
	profiling := false.
	(self menu itemTagged: #startProfiler) checked: false.! !

!QwaqServerUI methodsFor: 'services - delegated' stamp: 'hrs 1/31/2010 15:39'!
stopProxyServices
	"Stopthe proxy services (for mananging proxies for foreign clients)."
	serviceHolder stopProxyServices.
	(self menu itemTagged: #startProxyServices) checked: false.
! !

!QwaqServerUI methodsFor: 'services - delegated' stamp: 'jcg 11/11/2009 15:20'!
stopRouterServices
	"Stop the NetVid (streaming-video) services"
	serviceHolder stopRouterServices.
	(self menu itemTagged: #stopRouterServices) checked: true.
! !

!QwaqServerUI methodsFor: 'services' stamp: 'ar 9/9/2009 01:15'!
stopServiceProvider
	"Stop the front-end forum services"
	serviceProvider ifNil:[^self]. "not running"
	Transcript cr; show: 'Shutting down service provider'.
	serviceProvider destroy.
	serviceProvider := nil.

	(self menu itemTagged: #startServiceProvider) checked: false.

	"Do this last, because we want to log what goes on during shut-down."
	QLogger reset.
! !

!QwaqServerUI methodsFor: 'vnc support' stamp: 'ar 6/12/2007 18:05'!
stopVncServer
	"Launch the VNC server on the given port"
	vncServer ifNotNil:[
		vncServer stop.
		vncServer := nil.
	].
	(self menu itemTagged: #enableVNC) checked: false.! !

!QwaqServerUI methodsFor: 'services' stamp: 'hrs 12/4/2008 21:00'!
stopWebServices
	"Stop the Web server"
	QWebServices isRunning ifFalse: [^self]. "not running"
	QWebServices stop.
	(self menu itemTagged: #startWebServices) checked: false.
	"Do this last, because we want to log what goes on during shut-down."
	QLogger reset.! !

!QwaqServerUI methodsFor: 'services - delegated' stamp: 'jcg 11/25/2009 01:10'!
stopWebcastServices
	"Stop the RSTP broadcast services"
	serviceHolder stopWebcastServices.
	(self menu itemTagged: #startWebcastServices) checked: false.
! !

!QwaqServerUI methodsFor: 'menu' stamp: 'jcg 11/12/2009 12:50'!
toggleAppServices
	"Toggle the application services to be running"
	serviceHolder appServices 
		ifNil:[self startAppServices] 
		ifNotNil:[self stopAppServices].! !

!QwaqServerUI methodsFor: 'menu' stamp: 'ar 6/12/2007 12:11'!
toggleItem:aMenuItem forTool:aPanel

	"Callback for visibility/checkmark tracking for menu items; 
	 see my method #linkItem:toTool: "

	aMenuItem checked: (aPanel visible).! !

!QwaqServerUI methodsFor: 'menu' stamp: 'jcg 11/11/2009 15:22'!
toggleNetVidServices
	"Toggle the NetVid (streaming-video) services to be running"
	serviceHolder netVidServices 
		ifNil: [self startNetVidServices] 
		ifNotNil: [self stopNetVidServices].! !

!QwaqServerUI methodsFor: 'menu' stamp: 'ar 8/30/2007 13:36'!
toggleProfiler
	"Toggle profiler running"
	| response |
	profiling ifTrue:[
		self stopProfiling.
	] ifFalse:[
		response := CDialog request: 'Profiler log file' initialAnswer: profileLog.
		response ifNil:[^nil].
		profileLog := response.
		self startProfiling.
	].! !

!QwaqServerUI methodsFor: 'menu' stamp: 'hrs 1/31/2010 15:50'!
toggleProxyServices
	"Toggle the proxy services"
	serviceHolder toggleProxyServices ! !

!QwaqServerUI methodsFor: 'menu' stamp: 'jcg 11/11/2009 15:52'!
toggleRouterServices
	"Toggle the application services to be running"
	serviceHolder routerServices 
		ifNil: [self startRouterServices]
		ifNotNil: [self stopRouterServices].! !

!QwaqServerUI methodsFor: 'menu' stamp: 'ar 9/9/2009 01:16'!
toggleServiceProvider
	"Toggle the front-end services to be running"
	| response |
	serviceProvider ifNil:[
		response := CDialog request: 'Path to realms directory?' initialAnswer: realmsPath.
		response ifNil:[^nil].
		realmsPath := response.
		self startServiceProvider.
	] ifNotNil:[self stopServiceProvider].! !

!QwaqServerUI methodsFor: 'menu' stamp: 'ar 6/12/2007 22:10'!
toggleVncServer
	"Toggle VNC server status"
	| port |
	vncServer ifNil:[
		port := CDialog request: 'VNC port?' initialAnswer: '5900'.
		port ifNil:[^nil].
		port := port asInteger.
		self startVncServer: port.
	] ifNotNil:[
		self stopVncServer.
	].! !

!QwaqServerUI methodsFor: 'menu' stamp: 'hrs 12/4/2008 20:37'!
toggleWebServices
	"Toggle the web (qrl) services to be running"
	QWebServices isRunning ifTrue: [self stopWebServices] ifFalse: [self startWebServices].! !

!QwaqServerUI methodsFor: 'menu' stamp: 'jcg 11/25/2009 01:11'!
toggleWebcastServices
	"Toggle the NetVid (streaming-video) services to be running"
	serviceHolder webcastServices 
		ifNil:[self startWebcastServices] 
		ifNotNil:[self stopWebcastServices].! !

!QwaqServerUI methodsFor: 'menu' stamp: 'ar 7/29/2010 18:18'!
toolsMenu
	"Answer the tools menu"
	| menu item |
	menu := CMenu new.
	item := menu add: 'Network Activity...' target: trafficPane action: #toggle.
	self linkItem: item toTool: trafficPane.
	item := menu add: 'Latency Info...' target: latencyPane action: #toggle.
	self linkItem: item toTool: latencyPane.
	item := menu add: 'Transcript...' target: logView action: #toggle.
	self linkItem: item toTool: logView.
	item := menu add: 'VNC Server...' target: self action: #toggleVncServer.
	item tag: #enableVNC.
	^menu

! !

!QwaqServerUI methodsFor: 'accessing' stamp: 'bgf 11/1/2007 21:42'!
trafficPane
	"Answer the trafficPane of the receiver"
	<bewareOf: #trafficPaneChanged>
	^self propertyValueAt: #trafficPane! !

!QwaqServerUI methodsFor: 'accessing' stamp: 'bgf 11/1/2007 21:42'!
trafficPane: aValue
	"Modify the receiver's trafficPane"
	^self propertyValueAt: #trafficPane put: aValue with: #trafficPaneChanged! !

!QwaqServerUI methodsFor: 'status' stamp: 'ar 6/12/2007 12:01'!
update: aSymbol
	"Update from Transcript"
	[
		aSymbol == #appendEntry ifTrue:[self signal: #appendEntry with: Transcript contents].
	] on: Error do: [
		"Get it later."
	].! !

!QwaqServerUI methodsFor: 'error handling' stamp: 'ar 6/15/2007 11:01'!
userInterruptWatcher
	"Wait for user interrupts and open a notifier on the active process when one occurs."
	| interruptSemaphore |
	interruptSemaphore := InputSensor interruptSemaphore.
	[true] whileTrue: [
		interruptSemaphore wait.
		self signal: #userInterrupt.
	].! !

!QwaqServerUI methodsFor: 'accessing' stamp: 'ar 3/12/2008 15:09'!
usersPath
	"Answer the usersPath of the receiver"
	<bewareOf: #usersPathChanged>
	^self propertyValueAt: #usersPath! !

!QwaqServerUI methodsFor: 'accessing' stamp: 'ar 3/12/2008 15:09'!
usersPath: aValue
	"Modify the receiver's usersPath"
	^self propertyValueAt: #usersPath put: aValue with: #usersPathChanged! !

!QwaqServerUI methodsFor: 'initialize' stamp: 'ar 5/14/2011 14:15'!
versionFileContents
	"Answers the contents of the license file"
	| file data |
	file := [FileStream readOnlyFileNamed: 
			(QPropsFile default at: 'OpenQwaq.SP.VersionFile' ifAbsent:['/etc/OpenQwaq-release'])
	] on: Error do:[:ex|
		(ex isKindOf: FileDoesNotExistException) ifTrue:[^'No version file found'].
		^ex description
	].
	[data := file binary contents asString] ensure:[file close].
	^data! !

!QwaqServerUI methodsFor: 'accessing' stamp: 'ar 6/4/2009 14:28'!
vncPort
	"Answer the vncPort of the receiver"
	<bewareOf: #vncPortChanged>
	^self propertyValueAt: #vncPort! !

!QwaqServerUI methodsFor: 'accessing' stamp: 'ar 6/4/2009 14:28'!
vncPort: aValue
	"Modify the receiver's vncPort"
	^self propertyValueAt: #vncPort put: aValue with: #vncPortChanged! !

!QwaqServerUI methodsFor: 'accessing' stamp: 'ar 6/12/2007 18:02'!
vncServer
	"Answer the vncServer of the receiver"
	<bewareOf: #vncServerChanged>
	^self propertyValueAt: #vncServer! !

!QwaqServerUI methodsFor: 'accessing' stamp: 'ar 6/12/2007 18:02'!
vncServer: aValue
	"Modify the receiver's vncServer"
	^self propertyValueAt: #vncServer put: aValue with: #vncServerChanged! !

!QwaqServerUI methodsFor: 'accessing' stamp: 'jcg 11/25/2009 01:11'!
webcastServices
	^serviceHolder webcastServices
! !

Object subclass: #QServerLogToODBC
	instanceVariableNames: 'odbc queue logProcess'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Qwaq-Server-Logging'!

QServerLogToODBC class
	instanceVariableNames: 'default'!

!QServerLogToODBC commentStamp: '<historical>' prior: 0!
Super class for everything that should go straight to ODBC.!

QServerLogToODBC class
	instanceVariableNames: 'default'!

QServerLogToODBC subclass: #QClientActivityLoggerToODBC
	instanceVariableNames: 'activityQuery activityParams statsQuery statsParams infoQuery infoParams'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Qwaq-Server-Logging'!

!QClientActivityLoggerToODBC commentStamp: '<historical>' prior: 0!
Logs client records to ODBC.!

!QClientActivityLoggerToODBC class methodsFor: 'LICENSE' stamp: 'ar 4/20/2011 09:46'!
LICENSE
	^'Project OpenQwaq

	Copyright (c) 2005-20011, Teleplace, Inc., All Rights Reserved

	Redistributions in source code form must reproduce the above
	copyright and this condition.

	The contents of this file are subject to the GNU General Public
	License, Version 2 (the "License"); you may not use this file
	except in compliance with the License. A copy of the License is
	available at http://www.opensource.org/licenses/gpl-2.0.php.'! !

!QClientActivityLoggerToODBC methodsFor: 'initialize' stamp: 'ar 9/3/2008 21:25'!
initializeFrom: propsFile withPrefix: prefix

	(super initializeFrom: propsFile withPrefix: prefix, 'ClientActivityLog.') ifNil:[^nil].

	"Qwaq.SP.ClientActivityLog.ActivitiesQuery INSERT INTO client_activity_data_log (datetime_created, client_version, realm_name, username, status, activity_data) VALUES (%date, %version, %org, %user, 0, %contents)"
	self parseQuery: (propsFile at: prefix, 'ClientActivityLog.ActivitiesQuery')
		tokens: #(date version user org contents) 
		do:[:query :params| 
			activityQuery := query.
			activityParams := params.
		].

	"Qwaq.SP.ClientActivityLog.StatisticsQuery: INSERT INTO client_performance_data_log (datetime_created, client_version, realm_name, username, status, activity_data) VALUES (%date, %version, %org, %user, 0, %contents)"
	self parseQuery: (propsFile at: prefix, 'ClientActivityLog.StatisticsQuery')
		tokens: #(date version user org contents) 
		do:[:query :params| 
			statsQuery := query.
			statsParams := params.
		].

	"Qwaq.SP.ClientActivityLog.SysInfoQuery: INSERT INTO client_performance_data_log (datetime_created, client_version, realm_name, username, status, system_information) VALUES (%date, %version, %org, %user, 0, %contents)"
	self parseQuery: (propsFile at: prefix, 'ClientActivityLog.SysInfoQuery')
		tokens: #(date version user org contents) 
		do:[:query :params| 
			infoQuery := query.
			infoParams := params.
		].
! !

!QClientActivityLoggerToODBC methodsFor: 'client logs' stamp: 'ar 9/3/2008 16:14'!
record: xmlNode query: query args: params
	"Set up the ODBC request"
	| arguments arg |
	query ifNil:[^nil].
	arguments := params collect:[:each|
		each caseOf:{
			[#date]		->	[arg := DateAndTime now].
			[#version]	->	[arg := xmlNode attributeAt: 'client'].
			[#user]		->	[arg := xmlNode attributeAt: 'user'].
			[#org]		->	[arg := xmlNode attributeAt: 'orgName'].
			[#contents]	->	[arg := xmlNode contentString].
		}.
		arg].
	queue nextPut: (query -> arguments).! !

!QClientActivityLoggerToODBC methodsFor: 'client logs' stamp: 'ar 9/3/2008 21:17'!
recordActivities: xmlNode
	"Record client activities"
	^self record: xmlNode query: activityQuery args: activityParams! !

!QClientActivityLoggerToODBC methodsFor: 'client logs' stamp: 'ar 9/3/2008 21:17'!
recordStatistics: xmlNode
	"Record client statistics"
	^self record: xmlNode query: statsQuery args: statsParams! !

!QClientActivityLoggerToODBC methodsFor: 'client logs' stamp: 'ar 9/3/2008 21:17'!
recordSysInfo: xmlNode
	"Record client system information"
	^self record: xmlNode query: infoQuery args: infoParams! !

QServerLogToODBC subclass: #QExitQuizLoggerToODBC
	instanceVariableNames: 'surveyQuery surveyParams answerQuery answerParams'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Qwaq-Server-Logging'!

!QExitQuizLoggerToODBC commentStamp: '<historical>' prior: 0!
Logs exit quiz responses to ODBC. Also sends an email if a comment is entered.!

!QExitQuizLoggerToODBC class methodsFor: 'LICENSE' stamp: 'ar 4/20/2011 09:46'!
LICENSE
	^'Project OpenQwaq

	Copyright (c) 2005-20011, Teleplace, Inc., All Rights Reserved

	Redistributions in source code form must reproduce the above
	copyright and this condition.

	The contents of this file are subject to the GNU General Public
	License, Version 2 (the "License"); you may not use this file
	except in compliance with the License. A copy of the License is
	available at http://www.opensource.org/licenses/gpl-2.0.php.'! !

!QExitQuizLoggerToODBC methodsFor: 'initialize' stamp: 'ar 9/3/2008 21:49'!
initializeFrom: propsFile withPrefix: prefix

	(super initializeFrom: propsFile withPrefix: prefix, 'SurveyLog.') ifNil:[^nil].

	"Qwaq.SP.SurveyLog.SurveyQuery: INSERT INTO client_survey (datetime_created, client_version, realm_name, username, session_id, survey_name, client_comments) VALUES (%date, %version, %org, %user, %session, %survey, %comment)"
	self parseQuery: (propsFile at: prefix, 'SurveyLog.SurveyQuery')
		tokens: #(date version user org session survey comment) 
		do:[:query :params| 
			surveyQuery := query.
			surveyParams := params.
		].

	"Qwaq.SP.SurveyLog.AnswerQuery: INSERT INTO client_survey_answer (session_id, survey_question_id, client_question_answer, question_maximum_scale) VALUES (%session, %question %answer, %scale)"
	self parseQuery: (propsFile at: prefix, 'SurveyLog.AnswerQuery') 
		tokens: #(session question answer scale) 
		do:[:query :params| 
			answerQuery := query.
			answerParams := params.
		].
! !

!QExitQuizLoggerToODBC methodsFor: 'logging' stamp: 'ar 8/26/2009 17:06'!
recordExitSurvey: xmlNode
	"Record an exit quiz"
	| arguments arg comment |
	arguments := surveyParams collect:[:each|
		each caseOf:{
			[#date]		->	[arg := DateAndTime now].
			[#version]	->	[arg := xmlNode attributeAt: 'client'].
			[#user]		->	[arg := xmlNode attributeAt: 'user'].
			[#org]		->	[arg := xmlNode attributeAt: 'orgName'].
			[#session]	->	[arg := xmlNode contentStringAt: 'sessionId'].
			[#survey]	->	[arg := xmlNode contentStringAt: 'surveyName'].
			[#comment]	->	[arg := comment := xmlNode contentStringAt: 'clientComments'].
		}.
		arg].
	queue nextPut: (surveyQuery -> arguments).

	xmlNode tagsNamed: #surveyQuestions do:[:qNode|
		arguments := answerParams collect:[:each|
			each caseOf:{
				[#session]	->	[arg := xmlNode contentStringAt: 'sessionId'].
				[#question]	->	[arg := (qNode contentStringAt: 'questionId') asInteger].
				[#answer]	->	[arg := (qNode contentStringAt: 'questionAnswer') asInteger].
				[#scale]		->	[arg := (qNode contentStringAt: 'questionScale') asInteger].
			}.
			arg].
		queue nextPut: (answerQuery -> arguments).
	].
	"Comment is no longer mailed from here but rather from service provider;
	look at the sender(s) for this method to find it."
	comment.! !

!QServerLogToODBC class methodsFor: 'LICENSE' stamp: 'ar 4/20/2011 09:46'!
LICENSE
	^'Project OpenQwaq

	Copyright (c) 2005-20011, Teleplace, Inc., All Rights Reserved

	Redistributions in source code form must reproduce the above
	copyright and this condition.

	The contents of this file are subject to the GNU General Public
	License, Version 2 (the "License"); you may not use this file
	except in compliance with the License. A copy of the License is
	available at http://www.opensource.org/licenses/gpl-2.0.php.'! !

!QServerLogToODBC class methodsFor: 'accessing' stamp: 'ar 9/3/2008 21:05'!
default
	^default! !

!QServerLogToODBC class methodsFor: 'accessing' stamp: 'ar 9/3/2008 21:05'!
default: aLogger
	default := aLogger! !

!QServerLogToODBC class methodsFor: 'instance creation' stamp: 'ar 9/3/2008 21:05'!
newFrom: propsFile withPrefix: prefix
	"Create a new instance"
	^self new initializeFrom: propsFile withPrefix: prefix! !

!QServerLogToODBC class methodsFor: 'accessing' stamp: 'ar 9/3/2008 22:00'!
setupWithPrefix: prefix
	"Install the default logger with the given prefix"
	^self default: (self newFrom: QPropsFile default withPrefix: prefix)! !

!QServerLogToODBC methodsFor: 'initialize' stamp: 'ar 9/3/2008 21:03'!
initialize
	"Initialize the receiver"
	queue := SharedQueue new.
! !

!QServerLogToODBC methodsFor: 'initialize' stamp: 'ar 7/22/2010 22:45'!
initializeFrom: propsFile withPrefix: prefix
	"Note: Subclasses should override this to add the specific logger prefix"
	| dsn |
	dsn := propsFile at: prefix, 'DSN'.
	dsn ifEmpty:[^nil].
	"Use the provided data source for logging"
	odbc := (Smalltalk 
		at: #ODBCConnection 
		ifAbsent:[self error:'ODBC support is missing']) connectionClass new
			initializeDsn: dsn 
			user: (propsFile at: prefix, 'User') 
			password: (propsFile at: prefix, 'Password').
	[odbc connect] on: Error do:[:ex| QLogger logError: ex].
	self startLoggerProcess.! !

!QServerLogToODBC methodsFor: 'logging' stamp: 'ar 9/3/2008 21:03'!
odbcLog: query args: args

	odbc ifNil:[^nil].
	[^odbc execute: query args: args] on: Error do:[:ex| 
		QLogger logWarning: 'ODBC logging failed: ', ex description.
		ex return].
	"We only get here if the above fails"
	(Delay forSeconds: 1) wait.
	[true] whileTrue:[
		[odbc close; open.
		^odbc execute: query args: args] on: Error do:[:ex| 
			QLogger logError: ex.
			ex return].
		"We only get here if the above fails, too"
		(Delay forSeconds: 10*60) wait. "try again in ten minutes"
	].
! !

!QServerLogToODBC methodsFor: 'initialize' stamp: 'ar 9/3/2008 21:14'!
parseQuery: initialQuery tokens: tokens do: aBlock
	| map params query |
	initialQuery isEmptyOrNil ifTrue:[^self].
	map := tokens collect:[:str|
		(initialQuery findString: '%', str) -> str.
	].
	map := map sort reject:[:assoc| assoc key = 0].
	params := map collect:[:assoc| assoc value].
	query := params inject: initialQuery into:[:q :p| q copyReplaceAll: '%',p with: '?'].
	(query includes: $%) ifTrue:[self error: 'Invalid query (unresolved % arguments)'].
	aBlock value: query value: params.
! !

!QServerLogToODBC methodsFor: 'logging' stamp: 'ar 9/3/2008 21:04'!
runLoggerProcess
	"Run the logger process"
	| entry |
	[true] whileTrue:[
		entry := queue next.
		self odbcLog: entry key args: entry value.
	].! !

!QServerLogToODBC methodsFor: 'logging' stamp: 'ar 9/3/2008 21:04'!
startLoggerProcess
	self stopLoggerProcess.
	logProcess := self fork: #runLoggerProcess at: Processor userBackgroundPriority+1.! !

!QServerLogToODBC methodsFor: 'logging' stamp: 'ar 9/3/2008 21:04'!
stopLoggerProcess
	logProcess ifNotNil:[
		logProcess terminate.
		logProcess := nil.
	].! !

Object subclass: #QServiceHolder
	instanceVariableNames: 'serviceIP routerServices netVidServices appServices webcastServices internalHttpDispatcher externalHttpDispatcher clientDispatcher netVidRealmsPath maxApps streamManagers proxyServices httpsDispatcher'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Qwaq-Server-UI'!

!QServiceHolder commentStamp: '<historical>' prior: 0!
Holds services.  Introduced to remove replication of functionality in QwaqServerUI and QLocalServiceProvider.!

!QServiceHolder class methodsFor: 'LICENSE' stamp: 'ar 4/20/2011 09:46'!
LICENSE
	^'Project OpenQwaq

	Copyright (c) 2005-20011, Teleplace, Inc., All Rights Reserved

	Redistributions in source code form must reproduce the above
	copyright and this condition.

	The contents of this file are subject to the GNU General Public
	License, Version 2 (the "License"); you may not use this file
	except in compliance with the License. A copy of the License is
	available at http://www.opensource.org/licenses/gpl-2.0.php.'! !

!QServiceHolder methodsFor: 'accessing' stamp: 'jcg 11/11/2009 15:41'!
clientDispatcher
	^clientDispatcher! !

!QServiceHolder methodsFor: 'accessing' stamp: 'jcg 11/13/2009 18:19'!
externalHttpDispatcher
	^externalHttpDispatcher! !

!QServiceHolder methodsFor: 'initialize' stamp: 'jcg 11/12/2009 16:53'!
initialize
	super initialize.
	streamManagers := Dictionary new.! !

!QServiceHolder methodsFor: 'private' stamp: 'jcg 11/11/2009 15:41'!
installServicesInExternalHttpDispatcher
	externalHttpDispatcher connectionDispatcher: clientDispatcher
! !

!QServiceHolder methodsFor: 'private' stamp: 'jcg 11/13/2009 18:00'!
installServicesInInternalHttpDispatcher
	"XXXXX: we can't do this because we still don't manage QForumServices.  So, be sure that you start the http-dispatcher before you set up the forum-services (eg: in QwaqServerUI>>startSrerviceProvider)."
	"forumServices ifNotNil: [forumServices installServicesIn: internalHttpDispatcher]"! !

!QServiceHolder methodsFor: 'accessing' stamp: 'jcg 11/11/2009 15:46'!
internalHttpDispatcher
	^internalHttpDispatcher! !

!QServiceHolder methodsFor: 'initialize' stamp: 'jcg 11/12/2009 16:01'!
netVidRealmsPath: path
	"Hack for QLocalServiceProvider."
	netVidRealmsPath := path.! !

!QServiceHolder methodsFor: 'accessing' stamp: 'jcg 11/11/2009 15:06'!
netVidServices
	^netVidServices! !

!QServiceHolder methodsFor: 'private' stamp: 'ar 4/16/2011 17:18'!
rtspPort
	"The port the rtsp service listens on (554 is standard but < 1k so often not usable)"
	^Number readFrom: (QPropsFile default at: 'OpenQwaq.RTSP.Port' ifAbsent:['554'])! !

!QServiceHolder methodsFor: 'accessing' stamp: 'ar 11/22/2009 04:46'!
serviceIP: ipString
	serviceIP := ipString ifEmpty:[nil] ifNotEmpty:[
		NetNameResolver addressForName: ipString.
	].

! !

!QServiceHolder methodsFor: 'services' stamp: 'ar 4/16/2011 17:18'!
startAppServices
	"Start the app-services (for mananging TMS apps)."

	| mgr |
	appServices ifNotNil:[^self]. "already running"
	QLogger default baseName ifNil:[QLogger default baseName: 'appserver'].

	QLogger logInfo: 'Starting app services'.
	maxApps := (QPropsFile default at: 'OpenQwaq.Apps.MaxApps' ifAbsent:['40']) asNumber.
	mgr := self startStreamManagerOn: QServerConstants appServerPort.
	QAppServices reset.
	appServices := QAppServices default.
	appServices initializeApps: maxApps.
	mgr appServices: appServices.

	QLogger logInfo: 'Started app services'.! !

!QServiceHolder methodsFor: 'services' stamp: 'jcg 11/12/2009 14:36'!
startClientDispatcher
	"Start the TConnectionDispatcher for client connections"
	^clientDispatcher ifNil:[
		QLogger logInfo: 'Starting client dispatcher'.
		TConnectionDispatcher reset.
		clientDispatcher := TConnectionDispatcher default.
		clientDispatcher listenOn: 1234 interface: serviceIP.
		QLogger logInfo: 'Started client dispatcher'.

		"Reduce dependencies in order of service start-up."
		externalHttpDispatcher ifNotNilDo: [:d | d connectionDispatcher: clientDispatcher].
		routerServices ifNotNilDo: [:rs | rs dispatcher: clientDispatcher].

		clientDispatcher "answer it"
	].! !

!QServiceHolder methodsFor: 'services - http' stamp: 'ar 4/16/2011 17:18'!
startExternalHttpDispatcher
	"Start the internal HTTP dispatcher using default parameters."
	| hostString host ip httpPort |
	hostString := QPropsFile default at: 'OpenQwaq.SP.Web.InternalHostName'.
	host := hostString copyUpTo: $:.
	"If no web host has been given, use the primary service ip for the server"
	host ifEmpty:[host := QPropsFile default at: 'OpenQwaq.ServerIP'].
	ip := host ifEmpty: [nil] ifNotEmpty: [NetNameResolver addressForName: host].
	httpPort := ((hostString copyAfter: $:) 
		ifEmpty: [QServerConstants webPort printString]) asInteger.
	self startExternalHttpDispatcherOn: httpPort interface: ip.
! !

!QServiceHolder methodsFor: 'services - http' stamp: 'jcg 11/11/2009 14:36'!
startExternalHttpDispatcherOn: port interface: ip
	"Start the external HTTP dispatcher (used for public requests)."

	"Ensure we're not starting two dispatchers on the same port."
	internalHttpDispatcher ifNotNil: [:dispatcher |
		port = dispatcher port ifTrue: [
			self error: 'cannot use same port for internal and external HTTP dispatchers'
		].
	].

	"If we're already listening on a different port, don't start again.  Otherwise, tear it down."
	externalHttpDispatcher ifNotNilDo: [:dispatcher |
		dispatcher port = port ifTrue: [^self "nothing to do"].
		self stopExternalHttpDispatcher.
	].

	"Start the dispatcher."
	QLogger logInfo: 'Starting external HTTP dispatcher on port ', port.
	externalHttpDispatcher := HTTPServiceDispatcher new.
	externalHttpDispatcher listenOn: port interface: ip.
	externalHttpDispatcher accessLog: QLogger.
	externalHttpDispatcher errorLog: QLogger.
	
	self installServicesInExternalHttpDispatcher.

	QLogger logInfo: 'Started external HTTP dispatcher'.! !

!QServiceHolder methodsFor: 'services' stamp: 'ar 4/16/2011 17:18'!
startHttpsDispatcher
	"Set up https router access if we're configured to allow it."
	| port ip |
	self stopHttpsDispatcher.	
	
	(QPropsFile default at: 'OpenQwaq.Router.HTTPS') ifNotEmptyDo: [:spec |
		"Any non-empty value other than 'false' will start the service"
		spec = 'false' ifTrue: [^self].
		(spec includes: $:) ifTrue: [  "name:port"
			port := (spec copyAfter: $:) asNumber.
			ip := NetNameResolver addressForName: (spec copyUpTo: $:).
		] ifFalse: [  "e.g., 'true'"
			port := QServerConstants httpRouterPort.
			ip := serviceIP.
		].
		"We don't care where we are in startRouterServices, because we're only going to refer to
		it indirectly through the clientDispatcher."
		httpsDispatcher := HTTPSecureServiceDispatcher new
			accessLog: QLogger;
			errorLog: QLogger;
			connectionDispatcher: self startClientDispatcher;
			addFacetService: '/facet';
			listenOn: port interface: ip;
			yourself.
		QLogger logInfo: 'Started secure HTTP router dispatcher on ', ip, ':', port.
	].
! !

!QServiceHolder methodsFor: 'services - http' stamp: 'ar 4/16/2011 17:18'!
startInternalHttpDispatcher
	"Start the internal HTTP dispatcher using default parameters."
	| httpPort |
	httpPort := (QPropsFile default at: 'OpenQwaq.SP.HttpPort' 
					ifAbsent:[QServerConstants httpStatusPort]) asNumber.
	self startInternalHttpDispatcherOn: httpPort.
! !

!QServiceHolder methodsFor: 'services - http' stamp: 'jcg 11/11/2009 13:35'!
startInternalHttpDispatcherOn: port
	"Start the internal HTTP dispatcher (not public; used for administrative requests)."

	"Ensure we're not starting two dispatchers on the same port."
	externalHttpDispatcher ifNotNil: [:dispatcher |
		port = dispatcher port ifTrue: [
			self error: 'cannot use same port for internal and external HTTP dispatchers'
		].
	].

	"If we're already listening on a different port, don't start again.  Otherwise, tear it down."
	internalHttpDispatcher ifNotNilDo: [:dispatcher |
		dispatcher port = port ifTrue: [^self "nothing to do"].
		self stopInternalHttpDispatcher.
	].

	"Start the dispatcher."
	QLogger logInfo: 'Starting internal HTTP dispatcher on port ', port.
	internalHttpDispatcher := HTTPServiceDispatcher new.
	internalHttpDispatcher listenOn: port interface: serviceIP.
	internalHttpDispatcher accessLog: QLogger.
	internalHttpDispatcher errorLog: QLogger.
	
	self installServicesInInternalHttpDispatcher.

	QLogger logInfo: 'Started internal HTTP dispatcher'.! !

!QServiceHolder methodsFor: 'services' stamp: 'jcg 11/13/2009 12:25'!
startNetVidServices
	"Start the NetVid (streaming-video) services"
	| mgr |
	netVidServices ifNotNil:[^self]. "already running"
	QLogger default baseName ifNil:[QLogger default baseName: 'video'].

	QLogger logInfo: 'Starting NetVid services'.
	mgr := self startStreamManagerOn: QServerConstants netVidPort.
	QNetVidServices reset.
	netVidServices := QNetVidServices default.
	netVidServices realmsPath: netVidRealmsPath.
	netVidServices testConfiguration.
	mgr netVidServices: netVidServices.

	QLogger logInfo: 'Started NetVid services'.
! !

!QServiceHolder methodsFor: 'services' stamp: 'hrs 1/31/2010 16:37'!
startProxyServices
	"Start the proxy services (for mananging proxies for foreign clients)."

	| mgr |
	proxyServices ifNotNil:[^self]. "already running"
	QLogger default baseName ifNil:[QLogger default baseName: 'proxyserver'].

	mgr := self startStreamManagerOn: QServerConstants foreignClientProxyPort.
	QClientProxyServices reset.
	proxyServices := QClientProxyServices default.
	mgr proxyServices: proxyServices.
! !

!QServiceHolder methodsFor: 'services' stamp: 'hrs 2/4/2010 16:14'!
startRouterServices
	"Start the router services"
	| mgr |
	routerServices ifNotNil:[^self]. "already running"
	QLogger default baseName ifNil:[QLogger default baseName: 'router'].

	self startClientDispatcher.
	self startHttpsDispatcher.

	QLogger logInfo: 'Starting router services'.
	mgr := self startStreamManagerOn: QServerConstants routerPort.
	QRouterServices reset.
	routerServices := QRouterServices default.
	routerServices dispatcher: clientDispatcher.
	mgr routerServices: routerServices.
	QLogger logInfo: 'Started router services'.! !

!QServiceHolder methodsFor: 'services' stamp: 'ar 11/22/2009 04:51'!
startStreamManagerOn: port
	"Start the stream manager"
	| mgr |
	streamManagers at: port ifPresent: [:oldMgr |
		QLogger logWarning: 'stomping old stream-manager on port: ', port.
		oldMgr destroy.
	].
	mgr := QMediaStreamManager startOn: port interface: serviceIP.
	streamManagers at: port put: mgr.
	^ mgr! !

!QServiceHolder methodsFor: 'services' stamp: 'jcg 12/8/2009 13:09'!
startWebcastServices
	"Start the RTSP broadcast services"
	| mgr |
	webcastServices ifNotNil:[^self]. "already running"
	QLogger default baseName ifNil:[QLogger default baseName: 'webcast'].

	QLogger logInfo: 'Starting Webcast services'.
	mgr := self startStreamManagerOn: QServerConstants webcastPort.
	webcastServices := QWebcastServices new.
	self startExternalHttpDispatcher.
	webcastServices httpDispatcher: externalHttpDispatcher.
	"XXXXX: in our dash for the finish, regular RTSP (as opposed to RTSP-over-HTTP)
	 was almost certainly broken."
	"webcastServices rtspListenOn: self rtspPort."	
	mgr webcastServices: webcastServices.

	QLogger logInfo: 'Started Webcast services'.
! !

!QServiceHolder methodsFor: 'services' stamp: 'jcg 11/25/2009 01:10'!
stopAll
	self stopNetVidServices.
	self stopWebcastServices.
	self stopRouterServices.
	self stopAppServices.
	self stopInternalHttpDispatcher.
	self stopExternalHttpDispatcher.
	streamManagers do: [:sm | sm destroy].
	streamManagers := Dictionary new.! !

!QServiceHolder methodsFor: 'services' stamp: 'jcg 11/12/2009 12:47'!
stopAppServices
	"Stop the app-services (for mananging TMS apps)."
	appServices ifNil:[^self]. "not running"
	Transcript cr; show: 'Shutting down app services'.
	QAppServices reset.
	appServices := nil.

	"Do this last, because we want to log what goes on during shut-down."
	QLogger reset.! !

!QServiceHolder methodsFor: 'services - http' stamp: 'jcg 11/11/2009 13:30'!
stopExternalHttpDispatcher
	externalHttpDispatcher ifNotNilDo: [:dispatcher |
		QLogger logInfo: 'Stopping external HTTP dispatcher on port ', dispatcher port.
		externalHttpDispatcher := nil.
		dispatcher destroy.
	].! !

!QServiceHolder methodsFor: 'services' stamp: 'hrs 2/4/2010 16:03'!
stopHttpsDispatcher
	"Stops any https router dispatcher that may be running."
	httpsDispatcher ifNotNilDo: [:dispatcher |
		QLogger logInfo: 'Stopping HTTPS dispatcher on port ', dispatcher port.
		httpsDispatcher := nil.
		dispatcher destroy.
	].! !

!QServiceHolder methodsFor: 'services - http' stamp: 'jcg 11/11/2009 13:31'!
stopInternalHttpDispatcher
	internalHttpDispatcher ifNotNilDo: [:dispatcher |
		QLogger logInfo: 'Stopping internal HTTP dispatcher on port ', dispatcher port.
		internalHttpDispatcher := nil.
		dispatcher destroy.
	].! !

!QServiceHolder methodsFor: 'services' stamp: 'jcg 11/11/2009 15:11'!
stopNetVidServices
	"Stop the net-vid (streaming video) server"
	netVidServices ifNil:[^self]. "not running"
	Transcript cr; show: 'Shutting down NetVid services'.
	QNetVidServices reset.
	netVidServices := nil.

	"Do this last, because we want to log what goes on during shut-down."
	QLogger reset.! !

!QServiceHolder methodsFor: 'services' stamp: 'hrs 1/31/2010 15:35'!
stopProxyServices
	"Stop the TConnectionDispatcher for client connections."
	proxyServices ifNil:[^self]. "not running"
	QClientProxyServices reset.
	proxyServices := nil.

	"Do this last, because we want to log what goes on during shut-down."
	QLogger reset.! !

!QServiceHolder methodsFor: 'services' stamp: 'hrs 2/4/2010 16:04'!
stopRouterServices
	"Stop the router services"
	self stopHttpsDispatcher.
	routerServices ifNil:[^self]. "not running"
	Transcript cr; show: 'Shutting down router services'.
	routerServices destroy.
	routerServices := nil.

	"Do this last, because we want to log what goes on during shut-down."
	QLogger reset.! !

!QServiceHolder methodsFor: 'services' stamp: 'jcg 11/25/2009 01:16'!
stopWebcastServices
	"Stop the RSTP broadcast services"
	webcastServices ifNil:[^self]. "not running"
	Transcript cr; show: 'Shutting down Webcast services'.
	webcastServices destroy.
	webcastServices := nil.

	"Do this last, because we want to log what goes on during shut-down."
	QLogger reset.! !

!QServiceHolder methodsFor: 'services' stamp: 'hrs 1/31/2010 15:50'!
toggleProxyServices
	"Toggle the TConnectionDispatcher for client connections."
	proxyServices ifNil:[self startProxyServices] ifNotNil: [self stopProxyServices].! !

QLogger subclass: #QServerLogger
	instanceVariableNames: 'odbc activityQuery activityParams odbcDelay serverName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Qwaq-Server-Logging'!

!QServerLogger commentStamp: '<historical>' prior: 0!
QServerLogger emails error reports as a specific server policy.!

!QServerLogger class methodsFor: 'LICENSE' stamp: 'ar 4/20/2011 09:46'!
LICENSE
	^'Project OpenQwaq

	Copyright (c) 2005-20011, Teleplace, Inc., All Rights Reserved

	Redistributions in source code form must reproduce the above
	copyright and this condition.

	The contents of this file are subject to the GNU General Public
	License, Version 2 (the "License"); you may not use this file
	except in compliance with the License. A copy of the License is
	available at http://www.opensource.org/licenses/gpl-2.0.php.'! !

!QServerLogger methodsFor: 'private' stamp: 'ar 8/13/2008 20:04'!
appendEntry: e toFile: f

	"Inner workings of logging an entry;
	 write it to the supplied log file. "

	f nextPutAll: e asXml; nextPutAll: String crlf.
	(odbc notNil and:[e isActivityLogEntry]) ifTrue:[self odbcInsertActivity: e retry: 0].! !

!QServerLogger methodsFor: 'logging' stamp: 'ar 8/9/2008 22:17'!
log: aString
	"Don't allow unstructured logging"
	^self logInfo: aString from: '<unstructured>'.! !

!QServerLogger methodsFor: 'logging' stamp: 'ar 8/9/2008 22:18'!
log: aStringOrBlock from: anObject
	"Don't allow unstructured logging"
	^self logInfo: aStringOrBlock value from: anObject.! !

!QServerLogger methodsFor: 'log api' stamp: 'ar 4/16/2011 17:15'!
logError: desc details: debugInfo from: anObject
	"Log an error with associated detailed information."
	| email mail |
	email := QPropsFile default at: 'OpenQwaq.BugMail'.
	email ifNotEmpty:[
		mail := QwaqMail new.
		mail sentFromAddress: ((QPropsFile default at: 'OpenQwaq.SP.SupportEmail') 
			ifEmpty:['openqwaq@', serverName]).
		mail addAddresses: email.
		mail subject: 'Server Error [', serverName,']: ', desc.
		mail body: debugInfo.
		QwaqMailQueue default queueMail: mail.
	].
	super logError: desc details: debugInfo from: anObject.! !

!QServerLogger methodsFor: 'accessing' stamp: 'ar 8/12/2008 09:53'!
odbc
	"The activity log odbc connection"
	^odbc! !

!QServerLogger methodsFor: 'accessing' stamp: 'ar 8/12/2008 09:53'!
odbc: anODBCConnection
	"The activity log odbc connection"
	odbc := anODBCConnection! !

!QServerLogger methodsFor: 'private' stamp: 'ar 8/21/2008 10:47'!
odbcInsertActivity: e retry: count
	"Insert an activity record. If an error occurs, try to reopen the connection.
	Do this three times at most with a 1sec delay in the middle. If that doesn't
	work, log the error and retry five minutes from now."
	(odbcDelay notNil and:[odbcDelay < Time totalSeconds]) ifTrue:[
		"Start another attempt of recreating the ODBC connection"
		QLogger logWarning: 'ODBC reconnect attempt: ', (count+1). "zero based"
		[odbc close; open] on: Error do:[:ex|
			QLogger logWarning: 'ODBC Connect: ', ex description.
			count >= 3 ifTrue:[
				"We failed; delay recreation of the connection for another couple of minutes."
				QLogger logError: ex. "also sends mail"
				odbcDelay := Time totalSeconds + 300. "5 minutes"
			] ifFalse:[
				"Delay for a second then try again"
				(Delay forSeconds: 1) wait.
				self odbcInsertActivity: e retry: count+1.
			].
			^self].
		"We succeeded in reopening, reset the ODBC delay"
		odbcDelay := nil.
	].
	odbcDelay ifNil:[
		"Insert the record unless we're delaying because of errors"
		[odbc execute: activityQuery args: (activityParams collect:[:each| e perform: each])] 
			on: Error do:[:ex|
				QLogger logWarning: 'ODBC Insert: ', ex description.
				"If the query fails, try to recreate the connection in case that helps"
				count >= 3 ifTrue:[
					"We failed; delay recreation of the connection for another couple of minutes."
					QLogger logError: ex. "also sends mail"
					odbcDelay := Time totalSeconds + 300. "5 minutes"
				] ifFalse:[
					"Set the odbc delay in the past so that we retry once immediately."
					(Delay forSeconds: 1) wait.
					odbcDelay := Time totalSeconds - 1.
					self odbcInsertActivity: e retry: count+1
				].
			].
	].! !

!QServerLogger methodsFor: 'accessing' stamp: 'ar 12/9/2009 23:13'!
serverName
	"The name for the server we're running on."
	^serverName! !

!QServerLogger methodsFor: 'accessing' stamp: 'ar 12/9/2009 23:13'!
serverName: aString
	"The name for the server we're running on."
	serverName := aString! !

!QServerLogger methodsFor: 'initialize' stamp: 'ar 8/13/2008 00:45'!
setupActivityQuery: initialQuery
	"Set up query and arguments from the given string"
	| map |
	map := #(date host user action organization extra) collect:[:str|
		(initialQuery findString: '%', str) -> str.
	].
	map := map sort reject:[:assoc| assoc key = 0].
	activityParams := map collect:[:assoc| assoc value].
	activityQuery := activityParams inject: initialQuery into:[:q :p| q copyReplaceAll: '%',p with: '?'].
	(activityQuery includes: $%) ifTrue:[self error: 'Invalid query (unresolved % arguments)'].
! !
